﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Auxiliary;
using Cother;
namespace ImprovedXnaGame
{
    public partial class Session
    {
        public void UseCommand(Command c)
        {
            Item target = c.CommandObject as Item;
            if (!UtilizedCommands.Contains(c.ToString()))
                UtilizedCommands.Add(c.ToString());
            switch (c.CommandWord)
            {
                case "clear":
                    ConsoleOutput = new List<string>();
                    ConsoleTotalLines = 0;
                    ImprovedGame.Main.GreyOutputUntil = -1;
                    ImprovedGame.Main.ConsoleOutputCurrentLine = 0;
                    ImprovedGame.Main.PreviousConsoleTotalLines = 0;
                    break;
                case "toggle fullscreen":
                    Root.IsFullscreen = !Root.IsFullscreen;
                    break;
                case "help":
                    @WriteLines(@"This is a parser-based text adventure game. You may not be familiar with this type of game because they are mostly extinct for some twenty years. The principle is that you roleplay an adventure by typing commands and then pressing Enter. You will not need your mouse at all. 

To use a command, type in several letters from it (for example, start typing ""examine"" or ""door""). The suggestion system will automatically find appropriate commands for you to use. Choose among them with arrow keys and then press Enter to confirm your choice. It is impossible to lose the game or become stuck.

Most useful commands:
HELP - Shows this screen.
LOOK AROUND - Gives you a description of the room you're in usually with names of items you can interact with.
SAVE - Saves the game.
RESTORE - Loads last saved game.
EXAMINE - Gives a detailed description of an item or pony.
TALK TO - Initiates a conversation with a pony.

If you don't know what exactly you can do with an item, just type the item's name and a list of options will present itself to you.
Good luck saving Equestria from chaos!"); 
                    break;
                case "save":
                    ImprovedGame.Main.SavedSession = this;
                    if (Root.SaveSettings())
                        WriteLines("Session successfully saved.");
                    else
                        WriteLines("Save NOT successful. For some reason, I was unable to save your progress.");
                    break;
                case "restore":
                    if (Cother.ApplicationSettingsManagement.LoadSettings(ImprovedGame.Main) && ImprovedGame.Main.SavedSession != null)
                    {
                        ImprovedGame.Main.Session = ImprovedGame.Main.SavedSession;
                        ImprovedGame.Main.GreyOutputUntil = ImprovedGame.Main.Session.ConsoleTotalLines;
                        ImprovedGame.Main.PreviousConsoleTotalLines = ImprovedGame.Main.Session.ConsoleTotalLines;
                        ImprovedGame.Main.Session.WriteLines(@"Session restored. Type ""look around"" for a quick reminder of where you are.");
                        ImprovedGame.Main.ConsoleOutputCurrentLine = Math.Max(0, ImprovedGame.Main.Session.ConsoleTotalLines - ImprovedGame.Main.ConsoleMaxLines);
                        ImprovedGame.Main.Session.GetAvailableCommands(out ImprovedGame.Main.AvailableCommands, out ImprovedGame.Main.RecommendedCommands);
                    }
                    else
                    {
                        WriteLines("Cannot restore a saved game. Perhaps you didn't save any? (Save your progress using the \"save\" command.)");
                    }
                    break;
                case "exit":
                    ImprovedGame.Main.Exit();
                    break;
                case "look around":
                    WriteLines(GetLocationDescription(CurrentLocation));
                    break;
                case "examine":
                    if (c.CommandObject is Item)
                    {
                        Item i = c.CommandObject as Item;
                        WriteLines(i.Description);
                        if (i.ID == ItemID.Diamond1)
                        {
                            i.ID = ItemID.Diamond2;
                            i.Name = "Glittering Diamond";
                            i.Description = "It is a hard, beautiful and expensive diamond. I think you should eat it so the greedy unicorns won't take it away from you.";
                            i.PermittedCommands = new List<string>(new string[] { "eat", "pick up" });
                        }
                        if (i.ID == ItemID.Monitor_Western)
                        {
                            if (!i.PermittedCommands.Contains("Lockdown Time - choose"))
                            i.PermittedCommands.Add("Lockdown Time - choose");
                        }
                        if (i.ID == ItemID.Monitor_Northern)
                        {
                            if (!i.PermittedCommands.Contains("Call in Celestia - choose"))
                            i.PermittedCommands.Add("Call in Celestia - choose");
                        }
                        if (i.ID == ItemID.Monitor_Eastern)
                        {
                            if (!i.PermittedCommands.Contains("Empower Radiance - choose"))
                            i.PermittedCommands.Add("Empower Radiance - choose");
                        }
                        if (i.ID == ItemID.Monitor_Southern)
                        {
                            if (!i.PermittedCommands.Contains("Return Home - choose"))
                            i.PermittedCommands.Add("Return Home - choose");
                        }
                    }
                    break;
                case "pick up":
                    if (target != null)
                    {
                        Inventory.Add(target);
                        target.PermittedCommands.Remove("pick up");
                        CurrentLocation.Items.Remove(target);
                        WriteLines("You picked up " + target + ".");
                    }
                    break;
                case "inventory":
                    string inventorystring = "";
                    foreach (Item item in Inventory)
                    {
                        inventorystring += Environment.NewLine + item.Name;
                    }
                    WriteLines("Your inventory:" + inventorystring);
                    break;

                //----------------------------- GENERIC COMMANDS --------------------------------//
                case "talk to":
                    if (target != null)
                    {
                        if (( new ItemID[] { ItemID.Pony_Applejack, ItemID.Pony_Fluttershy, ItemID.Pony_PinkiePie, ItemID.Pony_RainbowDash, ItemID.Pony_Rarity, ItemID.Pony_Twilight }).Contains(target.ID))
                        {
                            bool thisIsNew = false;
                            if (Flag_SpokeToAtLeastOnePony)
                            {

                            }
                            else
                            {
                                WriteLines(@"You begin by introducing yourself, ""Hello, everypony, I am -""

""Star Swirl the Bearded"", fills in the violet unicorn. ""I very much look forward to talking to you. You're only the most important conjuror of the preclassical era. There must be tons of spells you can teach --""

""Perhaps we should just concentrate on getting behind that solid door, there?"" the orange earth pony interrupts her.

""Yes,"" you say, ""wait, I am called The Bearded in the future? That's... nevermind.""");
                                Flag_SpokeToAtLeastOnePony = true;
                                thisIsNew = true;
                            }
                            if (target.ID == ItemID.Pony_Applejack)
                            {
                                if (Inventory.Find((Item j) => { return j.ID == ItemID.Rope; }) != null)
                                {
                                    if (!Flag_ApplejackSpokenTo)
                                    {
                                         WriteLines(@"""Name's Applejack, sir,"" the pony said, ""you give me a rope, I could probably use it -- oh, you already have it.""", thisIsNew);
                                    }
                                    Inventory.RemoveAll((Item j) => { return j.ID == ItemID.Rope; });
                                    WriteLines(@"""Yes, this should do nicely.""

Applejack grabs the rope in her mouth, takes a running start and jumps across the river. She nearly doesn't make it but in the end, she manages to climb to solid ground. With a skill you have not seen before, she throws the rope over, where it effortlessly binds around the pink pony who screams ""YAAAAAY!!! BEST ADVENTURE EVER!!"" as she is pulled onto the other side.", !Flag_ApplejackSpokenTo || thisIsNew);
                                    Flag_ApplejackSpokenTo = true;
                                    CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.Pony_Applejack || j.ID == ItemID.Pony_PinkiePie; });
                                    Flag_PinkieAndApplejackAcross = true;
                                }
                                else
                                {
                                    WriteLines(@"""Name's Applejack, sir,"" the pony said, ""you give me a rope, I could probably use it bring at least one pony on the otherside, and myself too.""", thisIsNew);
                                    Flag_ApplejackSpokenTo = true;
                                }
                              
                            }
                            if (target.ID == ItemID.Pony_Fluttershy)
                            {
                                WriteLines(@"""Who are you, lady?"" you ask, but the pony just shakes her head and does not seem to want to respond. If I were you, I'd take it as an offense and throw her into the river. Also, something prevents you from teleporting her across the river.", thisIsNew);
                                Flag_FluttershySpokenTo = true;
                                if (Flag_FluttershySpokenTo && (Flag_RainbowDashSpokenTo || Flag_TwilightSpokenTo) && !target.PermittedCommands.Contains("encourage (I believe in you.)"))
                                {
                                    target.PermittedCommands.Add("encourage (I believe in you.)");
                                    target.PermittedCommands.Add("encourage (There is no other way.)");
                                    target.PermittedCommands.Add("encourage (I will give you cake.)");
                                    target.PermittedCommands.Add("encourage/lie (The river is just illusory.)");
                                }
                            }
                            if (target.ID == ItemID.Pony_PinkiePie)
                            {
                                WriteLines(@"""HELLO!"" shouts the pink pony, ""are you afraid of dying? Cause I'm afraid of dying, you know, with the lava rising and not stopping and threatening to soon engulf and burn us all so we will all die soon, you know, like, we will be INCINERATED! You like that word? Okay, oh, and do you like CUPCAKES? Do you have cupcakes in the past?""

You see that even if she jumps, she won't be able to get across the river.", thisIsNew);
                                Flag_PinkieSpokenTo = true;
                            }
                            if (target.ID == ItemID.Pony_RainbowDash)
                            {
                                WriteLines(@"""I'm Rainbow Dash, the coolest pegasus from the future. So, how do we defeat Discord?"" she says.

""Do you think you could fly your friends over the river?"" you ask.

""Perhaps, but only with Fluttershy's help, and only one. In this heat, I would be far too tired to transport another one. And I don't think Fluttershy will be brave enough to do this. You might have to encourage her.""", thisIsNew);
                                Flag_RainbowDashSpokenTo = true;
                                Item fluttershy = CurrentLocation.Items.Find((Item j) => { return j.ID == ItemID.Pony_Fluttershy; });
                                if (Flag_FluttershySpokenTo && (Flag_RainbowDashSpokenTo || Flag_TwilightSpokenTo) && !fluttershy.PermittedCommands.Contains("encourage (I believe in you.)"))
                                {
                                    fluttershy.PermittedCommands.Add("encourage (I believe in you.)");
                                    fluttershy.PermittedCommands.Add("encourage (There is no other way.)");
                                    fluttershy.PermittedCommands.Add("encourage (I will give you cake.)");
                                    fluttershy.PermittedCommands.Add("encourage/lie (The river is just illusory.)");
                                }
                            }
                            if (target.ID == ItemID.Pony_Rarity)
                            {
                                WriteLines(@"""My name is Rarity, sir,"" says the unicorn, ""and we would be ever so grateful if you could help us from this ascending column of fiery lava. And quickly, if such a thing is possible, please.""

You sense she won't be able to get across without help.", thisIsNew);
                                Flag_RaritySpokenTo = true;
                            }
                            if (target.ID == ItemID.Pony_Twilight)
                            {
                                WriteLines(@"""I'm extremely honored to meet you, sir,"" says the unicorn, ""I am Twilight Sparkle.""

""Do you have any ideas, Miss Sparkle?"" you ask. She points her hoof to the orange pony, ""if you conjure a rope, she could use it get herself and Pinkie across. You and I can teleport ourselves. And then, there must be some way that the two pegasi transport Rarity. But of course, you lead. You're much more experienced than I am.""

'Perhaps I am', you think, 'though you must be happier seeing that you have five friends.'", thisIsNew);
                                Flag_TwilightSpokenTo = true;
                                Item fluttershy = CurrentLocation.Items.Find((Item j) => { return j.ID == ItemID.Pony_Fluttershy; });
                                if (Flag_FluttershySpokenTo && (Flag_RainbowDashSpokenTo || Flag_TwilightSpokenTo) && !Flag_FluttershyIsReady && !fluttershy.PermittedCommands.Contains("encourage (I believe in you.)"))
                                {
                                    fluttershy.PermittedCommands.Add("encourage (I believe in you.)");
                                    fluttershy.PermittedCommands.Add("encourage (There is no other way.)");
                                    fluttershy.PermittedCommands.Add("encourage (I will give you cake.)");
                                    fluttershy.PermittedCommands.Add("encourage/lie (The river is just illusory.)");
                                }
                            }
                        }
                        if (target.ID == ItemID.CloverPhase1)
                        {
                            if (Flag_Step2Done && !Flag_Step3Done)
                            {
                                WriteLines(@"You say, ""Em, Clover? Could you lend me a hoof here please?"".

""Anything, Star. What do you need?"" replies the unicorn.

""Just put the contents of this violet flask into the magic bowl, then I'll summon the avatar.""

As Clover pours in the violet liquid, the solution begins to sparkle.");
                                Flag_Step3Done = true;
                                CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.VioletFlask; });
                            }
                            else
                            {
                                WriteLines(@"""I really hope you'll succeed, Star. Because if the avatar can't help us, pegasi might be beyond our help,"" says Clover.

""Don't give up so quickly, Clover. Even if I fail, there's still a lot we can do. I mean, the pegasi have wings. They could just outfly the storm and they'll only lose the buildings.""");
                            }
                        }
                    }
                    break;
                case "read":
                    if (c.CommandObject is Item)
                    {
                        Item i = c.CommandObject as Item;
                        if (i.ID == ItemID.UnicorniaDaily)
                        {
                            WriteLines(@"Equestria Daily: Chaos Storm unlike any seen before approaches Pegatlantis.

We've all seen storms of chaos before. They appear erratically and unpredictably, wreaking havoc wherever they come. Fortunately for us, these storms are usually localized and after a few hours, they generally disappear, leaving behind a landscape in dire need of repairs.

However, nopony has ever seen before a storm like the one that's currently approaching Pegatlantis. It took form near the ocean a few days (!) ago and unless stopped, it threatens to destroy Pegatlantis completely.

Our specialist on harmony, Mr. Allegro, says that this defies all logic. Discord should not have been capable of producing a storm of this magnitude. 

All ponies near the storm are requested to stay at homes and not panic.");
                        }
                        if (i.ID == ItemID.Spellbook)
                        {
                            WriteLines(@"To summon the avatar of friendship, follow these steps in this exact order:

1. Pour any blue solution into a magic bowl.
2. Pour any orange solution into the magic bowl.
3. Ask a friend to pour any violet solution into the magic bowl.
4. Cast a summoning spell.");
                            if (!Flag_SpellbookRead)
                            {
                                Flag_SpellbookRead = true;
                                CurrentLocation.Items.Find((Item j) => { return j.ID == ItemID.BlueFlask; }).PermittedCommands.Add("pour in");
                                CurrentLocation.Items.Find((Item j) => { return j.ID == ItemID.OrangeFlask; }).PermittedCommands.Add("pour in");
                                CurrentLocation.Items.Find((Item j) => { return j.ID == ItemID.VioletFlask; }).PermittedCommands.Add("pour in");
                                CurrentLocation.Items.Find((Item j) => { return j.ID == ItemID.GreenFlask; }).PermittedCommands.Add("pour in");
                                CurrentLocation.Items.Find((Item j) => { return j.ID == ItemID.CloverPhase1; }).PermittedCommands.Add("talk to");
                            }
                        }
                    }
                    break;
                //--------------------------- ONE TIME SPECIFIC COMMANDS ---------------------------------------//
                case "change my choice":
                    EnterLocation(Locations[LocationID.NarrativeCore]);
                    break;
                case "cast fateseal spell":
                    EnterLocation(Locations[LocationID.TheEnd]);
                    break;
                case "Lockdown Time - choose":
                    Flag_ChosenFuture = FutureID.LockdownTime;
                    EnterLocation(Locations[LocationID.FinalChoice]);
                    break;
                case "Call in Celestia - choose":
                    Flag_ChosenFuture = FutureID.CallInCelestia;
                    EnterLocation(Locations[LocationID.FinalChoice]);
                    break;
                case "Empower Radiance - choose":
                    Flag_ChosenFuture = FutureID.EmpowerRadiance;
                    EnterLocation(Locations[LocationID.FinalChoice]);
                    break;
                case "Return Home - choose":
                    Flag_ChosenFuture = FutureID.ReturnHome;
                    EnterLocation(Locations[LocationID.FinalChoice]);
                    break;
                case "enter the Narrative Core":
                    EnterLocation(Locations[LocationID.NarrativeCore]);
                    break;
                case "press button 1":
                    WriteLines(@"A lightning bolt strikes you from the door and you fall over. When you get back on your hooves, you find that you are unharmed. Your personal protective spells hold.");
                    break;
                case "press button 2":
                    DoorCombination(4, 5);
                    break;
                case "press button 3":
                    DoorCombination(5, 4);
                    break;
                case "press button 4":
                    DoorCombination(1, 5);
                    break;
                case "press button 5":
                    DoorCombination(5, 1);
                    break;
                case "press button 6":
                    DoorCombination(2, 4);
                    break;
                case "press button 7":
                    DoorCombination(6, 0);
                    break;
                case "press button 8":
                    DoorCombination(3, 4);
                    break;
                case "press button 9":
                    DoorCombination(2, 3);
                    break;
                case "press button 10":
                    DoorCombination(2, 1);
                    break;
                case "press button 11":
                    DoorCombination(6, 5);
                    break;
                case "press button 12":
                    DoorCombination(3, 4);
                    break;
                case "press button 13":
                    WriteLines(@"A ball of fire strikes you from the door and you fall over. When you extinguish your clothes, you find that, fortunately, you remain unharmed.");
                    break;
                case "press button 14":
                    DoorCombination(1, 2);
                    break;
                case "press button 15":
                    DoorCombination(5, 6);
                    break;
                case "press button 16":
                    LockDoor(1);
                    LockDoor(2);
                    LockDoor(3);
                    LockDoor(4);
                    LockDoor(5);
                    LockDoor(6);
                    WriteLines(@"You have reset the mechanism. All the doors are now locked.");
                    break;
                case "send Fluttershy, Rainbow Dash and Rarity over":
                    WriteLines(@"Fluttershy and Rainbow Dash both grab Rarity and gently fly her over the river to the other side. They encounter no problems and Fluttershy is obviously relieved.");
                    Flag_FluttershyRainbowRarityOver = true;
                    break;
                case "teleport over and tell Twilight to do the same":
                    WriteLines(@"Your horns briefly emit a purple light and then you both find yourself across the lava river.");
                    Flag_StarSwirlTwilightOver = true;
                    CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.Pony_Twilight; });
                    break;
                case "encourage (I believe in you.)":
                    WriteLines(@"""I know you can do this, Fluttershy,"" you comfort her, ""you know you are strong and would do anything for your friends. Knowing that, you cannot fail.""

She looks up to you, stares at you for a while and then, silently, she nods. Quietly she whispers: ""I'll try.""

""That's the spirit!"" shouts Rainbow Dash, ""you tell us to go when ready.""");
                    Flag_FluttershyIsReady = true;
                    CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.Pony_RainbowDash || j.ID == ItemID.Pony_Fluttershy || j.ID == ItemID.Pony_Rarity; });
                    break;
                case "encourage (There is no other way.)":
                    WriteLines(@"""You don't really have a choice, Fluttershy"" you inform her, ""if you can't help Rainbow fly Rarity over, she will remain here and then --""

She looks up to you, ""but...""

""There's really no other way, Fluttershy,"" says Rainbow Dash, ""so get up here and help me.""

After a while she stands up, decided. ""I will do this! I could never look at myself again if I left Rarity behind.""");
                    Flag_FluttershyIsReady = true;
                    CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.Pony_RainbowDash || j.ID == ItemID.Pony_Fluttershy || j.ID == ItemID.Pony_Rarity; });
                    break;
                case "encourage (I will give you cake.)":
                    WriteLines(@"""If you help get Rarity over"" you tell her, ""I will give you some delicious and moist cake!""

You realize very quickly this avenue won't work. Fluttershy simply shakes her head and remains silent.");
                    break;
                case "encourage/lie (The river is just illusory.)":
                    WriteLines(@"""Actually, my magic detects the river is but an illusion,"" you lie, ""even if you let go of Rarity, she will be safe.""

You nod to Twilight Sparkle so she remains silent. ""I am absolutely sure about this.""

""In that case,"" says Fluttershy, ""I'm ready.""");
                    Flag_FluttershyIsReady = true;
                    CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.Pony_RainbowDash || j.ID == ItemID.Pony_Fluttershy || j.ID == ItemID.Pony_Rarity; });
                    break;
                case "conjure rope":
                    WriteLines(@"With but a thought, you conjure a long rope. It lies beneath your hooves on the cave floor. You still have to pick it up.");
                    CurrentLocation.Items.Add(new Item(ItemID.Rope, "Long Rope", "It has been suggested to you that Applejack might jump to the other side and then use this rope to bring across one of her friends.", new string[] { "pick up" }));
                    Flag_RopeConjured = true;
                    break;
                case "go downwards":
                    if (Flag_StarSwirlTwilightOver)
                    {
                        WriteLines(@"You lock the solid doors behind yourself as you descend and can feel how the rising lava smashes against them. But the doors hold and you are safe. Then, all of sudden, your six visitors from the future disappear only to be replaced by - you guessed it - me!

""Oh, it would be too easy from now on,"" I mock you, ""plus, I have a little labyrinth prepared for them in the future. As for you, your only way outside is cut. So, enjoy being buried in the middle of Equestria under miles of rock. Me, I have some chaos to wreak.""");
                        EnterLocation(Locations[LocationID.FinalDoor], true);
                    }
                    else
                    {
                        WriteLines(@"When you walk down the stairwell behind the metal door, you enter a huge room.");
                        EnterLocation(Locations[LocationID.LavaRoom], true);
                        WriteLines(@"Then I materialize in front of you, six ponies behind me. ""Hello again, old man,"" I greet you, ""I thought I'd bring you these six fellas from the future. They were somewhat interfering with my plans there. Oh, and may I mention the fact that the lava is rising?"" 

I smile from ear to ear, with but a gesture destroy the upward-leading stairwell and disappear in a puff of smoke.", true);
                        Flag_EnteredLavaRoom = true;
                    }
                    break;
                case "think upon Clover":
                    WriteLines(@"'I have dedicated my entire life to the study of magic and invention of spells. This way I have helped countless ponies, improved their life and made them happy. But in the end, it was the time I spent with Clover that I enjoyed the most. Which brings me to the one question I don't know answer for: was all the study worth it? If I made friends rather than push the boundaries of knowledge, would I have lived a happier life? And would that be a good thing?'");
                    Flag_ThoughtUpon1 = true;
                    break;
                case "eat":
                    WriteLines(@"If I were mean, I'd let you do it. But because I'm nice like that, I'll just tell you that eating a sharp diamond isn't the best idea ever. Also, it would make for an awfully short and boring story.");
                    break;
                case "open":
                    WriteLines(@"It is locked and sealed. Your feeble old hands cannot open it.");
                    break;
                case "unlock":
                    WriteLines(@"You cast an Alohomora spell, but the door is magically warded. The unicorns who made it probably wanted nothing to come to the surface from the depths.");
                    break;
                case "break":
                    if (Inventory.Find((Item j) => { return j.ID == ItemID.Diamond2; }) != null)
                    {
                        WriteLines(@"You cast a fast-rotation spell on the diamond and use it as a chainsaw. The doors crack open and reveal a dark passage leading downwards.");
                        Flag_DoorDestroyed = true;
                        CurrentLocation.Description = @"You are in a small pony-made cavern without sunlight. There used to be an upwards-leading tunnel but it seems to have been destroyed on purpose.";
                        Inventory.RemoveAll((Item j) => { return j.ID == ItemID.Diamond2; });
                        CurrentLocation.Items.Remove(target);
                    }
                    else
                    {
                        WriteLines(@"That's a good idea, but your spells nor your physical strength (or should I say, weakness) are not sufficient to open it. You need a sharp or hard object.");
                    }
                    break;
                case "cast sense-life spell":
                    WriteLines(@"Nopony seems to be in your immediate vicinity. Clover was most definitely not teleported with you and may now be at my mercy -- then again, you all are.");
                    break;
                case "cast light spell":
                    WriteLines(@"Your horn begins to glow and illuminates the entire area.");
                    Flag_CastLightSpell = true;
                         CurrentLocation.Description = @"You are in a small pony-made cavern without sunlight. There used to be an upwards-leading tunnel but it seems to have been destroyed on purpose. Something is written on the stone walls."; 
                         CurrentLocation.Items.Add(new Item(ItemID.Writings, "Writings", @"To all unicorns: This section of the mines contains a minimal amount of gemstones and is much too close to the core. It is now condemned and sealed."));
                    CurrentLocation.Items.Add(new Item(ItemID.Diamond1, "Glittering Object", @"It is a polished diamond, very beautiful and very expensive. Unicorns will likely want to take it away from you. Perhaps you want to eat it so nopony else can have it?"));
                    CurrentLocation.Items.Add(new Item(ItemID.MetalDoor, "Metal Door", @"It seems to lead further down, however, it is sealed shut.", new string[] { "open", "unlock", "break" }));
                    WriteLines(GetLocationDescription(CurrentLocation), true);
                    break;
                case "begin":
                    WriteLines(@"You are Star Swirl the Bearded. Equestria has just been founded but is in a state of turmoil under Discord's rule. As Discord's interventions continue to intensify, you decide to call upon your most powerful spell and summon the personification of Friendship to advise you and the rulers of Equestria.
");
                    EnterLocation(Locations[LocationID.Laboratory], true);
                    break;
                case "cast weather-control spell":
                    WriteLines(@"You cast a storm-cancelling spell of your own invention but it fizzles.

" + DiscordAppearance1);
                    EnterLocation(Locations[LocationID.DarkCave], true);
                    break;
                case "teleport to the village":
                    WriteLines(@"The force field may not be strong enough to withhold the storm, but it is sufficient to prevent your teleportation attempts. You think of another way to get in.

" + DiscordAppearance1);
                    EnterLocation(Locations[LocationID.DarkCave], true);
                    break;
                case "empower the unicorns' force field":
                    WriteLines(@"The force field grows stronger. Perhaps now it will hold and protect the unicorns. You think about what to do next.

" + DiscordAppearance1);
                    EnterLocation(Locations[LocationID.DarkCave], true);
                    break;
                case "comfort Clover the Clever":
                    WriteLines(@"""I'm positive the force field will hold,"" you lie, ""unicorns are strong."" 

You don't even mention the fact that the unicorns inside will probably turn against each other soon.");
                    Flag_ComfortedClover = true;
                    break;
                    // TODO repeated Comfort

                case "go outside":
                    EnterLocation(Locations[LocationID.Outside]);
                    break;
                case "cast summoning spell":
                    if (Flag_Step3Done)
                    {
                        WriteLines(@"The magic bowl transforms into a sphere of pure energy which then takes shape of a small earth pony.

""I am Radiance, the personification of friendship,"" she says, ""a direct opposite of Discord. Now, listen to me, Star Swirl. Very soon, things will no longer make sense, anywhere and anytime. Discord's powers grow in the future and by extension, now. Whatever happens, you must go to the center of the world or as near as possible and locate the Narrative Core. You will understand later.

""But for now, remember this: Find the Narrative Core. Nothing else matters. If you fail, Discord's powers will grow limitless and he will never be stopped. For the sake of everypony, you must succeed.""

Radiance doesn't wait for a reply and disappears into thin air. At that point, Clover screams at you: ""Star, come outside, quick. You must see this!""");
                        Flag_Step4Done = true;
                        CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.CloverPhase1; });
                    }
                    else
                    {
                        WriteLines("Nothing happens.");
                    }
                    break;
              
                case "pour in":
                    if (target != null)
                    {
                        if (target.ID == ItemID.BlueFlask)
                        {
                            CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.BlueFlask; });
                            Flag_Step1Done = true;
                            WriteLines("You have poured the blue solution into the magic bowl.");
                        }
                        if (target.ID == ItemID.OrangeFlask)
                        {
                            if (Flag_Step1Done)
                            {
                                CurrentLocation.Items.RemoveAll((Item j) => { return j.ID == ItemID.OrangeFlask; });
                                Flag_Step2Done = true;
                                WriteLines("You have poured the orange solution into the magic bowl. It has taken on all colors of the rainbow.");
                            }
                            else
                            {
                                WriteLines("I wouldn't do that if I ever you. Though, if you want to doom everypony, go ahead and pour it in.");
                            }
                        }
                        if (target.ID == ItemID.GreenFlask)
                        {
                            WriteLines("I wouldn't do that if I ever you. Though, if you want to doom everypony, go ahead and pour it in.");
                        }
                        if (target.ID == ItemID.VioletFlask)
                        {
                            WriteLines("While I am sure you just love yourself, I don't think the magic will consider you to be your friend.");
                        }
                    }
                    break;
                

            }
        }

        private void DoorCombination(int unlock, int lockdoor)
        {
            UnlockDoor(unlock);
                if (lockdoor != 0)
            LockDoor(lockdoor);
             if (Flag_DoorUnlocked_1 && Flag_DoorUnlocked_2 && Flag_DoorUnlocked_3 && Flag_DoorUnlocked_4 && Flag_DoorUnlocked_5 && Flag_DoorUnlocked_6)
            {
                Flag_AllDoorsOpen = true;
            }

             WriteLines("You have unlocked door number " + unlock + ".");
                if (lockdoor != 0)
            WriteLines("You have locked door number " + lockdoor + ".", true);
            WriteLines("Currently, open doors are: " +
                (Flag_DoorUnlocked_1 ? "1 " : "") +
                (Flag_DoorUnlocked_2 ? "2 " : "") +
                (Flag_DoorUnlocked_3 ? "3 " : "") +
                (Flag_DoorUnlocked_4 ? "4 " : "") +
                (Flag_DoorUnlocked_5 ? "5 " : "") +
                (Flag_DoorUnlocked_6 ? "6 " : ""), true);
            if (Flag_AllDoorsOpen)
            {
                WriteLines(@"The gate before you slides to the side, then the gate behind, then again and again and again and again. When all six gates stand open, you may enter the Narrative Core and finally discover what it is Radiance wanted you to find. 

I know that I am beaten. I knew from the outset that Discord could never really triumph over Friendship. You, Star Swirl, have managed to pass tests of cooperation, ingenuity and skill. When you enter the core and pick your choice, know that I am happy to have been defeated by such a clever foe.", true);
                CurrentLocation.Description = "Before you are six open gates and the way is free to enter the Narrative Core.";
            }           
        }

        private void LockDoor(int lockdoor)
        {
            switch (lockdoor)
            {
                case 1: Flag_DoorUnlocked_1 = false; break;
                case 2: Flag_DoorUnlocked_2 = false; break;
                case 3: Flag_DoorUnlocked_3 = false; break;
                case 4: Flag_DoorUnlocked_4 = false; break;
                case 5: Flag_DoorUnlocked_5 = false; break;
                case 6: Flag_DoorUnlocked_6 = false; break;
            }
        }

        private void UnlockDoor(int unlock)
        {
            switch (unlock)
            {
                case 1: Flag_DoorUnlocked_1 = true; break;
                case 2: Flag_DoorUnlocked_2 = true; break;
                case 3: Flag_DoorUnlocked_3 = true; break;
                case 4: Flag_DoorUnlocked_4 = true; break;
                case 5: Flag_DoorUnlocked_5 = true; break;
                case 6: Flag_DoorUnlocked_6 = true; break;
            }
        }

        public const string DiscordAppearance1 = @"Then, suddenly, I materialize myself in front of your very eyes.

""Discord!"" you say, ""this is insane. It's impossible for you to have this much power.""

I smile, ""Insane? That I like. Impossible? Don't know the word. You see, some time in the future, three little fillies have given me all the power I needed. And so I've decided to take a little trip through time,"" I look around, ""I don't know how you can like it in here. You know what? I'll find you a better place.""
There's a flash of light.
";
        public bool Flag_DoorDestroyed;
        public bool Flag_EnteredLavaRoom;
        public bool Flag_ApplejackSpokenTo;
        public bool Flag_RaritySpokenTo;
        public bool Flag_PinkieSpokenTo;
        public bool Flag_TwilightSpokenTo;
        public bool Flag_RainbowDashSpokenTo;
        public bool Flag_FluttershySpokenTo;
        public bool Flag_RopeConjured;

        public bool Flag_FluttershyIsReady;

        public bool Flag_PinkieAndApplejackAcross;
        public bool Flag_FluttershyRainbowRarityOver;
        public bool Flag_StarSwirlTwilightOver;
        public bool Flag_DoorUnlocked_1;
        public bool Flag_DoorUnlocked_2;
        public bool Flag_DoorUnlocked_3;
        public bool Flag_DoorUnlocked_4;
        public bool Flag_DoorUnlocked_5;
        public bool Flag_DoorUnlocked_6;
        public bool Flag_AllDoorsOpen;
    }
    public partial class ImprovedGame : Microsoft.Xna.Framework.Game
    {
        public List<Command> AvailableCommands = new List<Command>();
        public List<Command> RecommendedCommands = new List<Command>();
        public void SendCommand(Command c)
        {
            Session.UseCommand(c);
            Session.GetAvailableCommands(out AvailableCommands, out RecommendedCommands);
        }
    }
    public class Command
    {
        public string CommandWord;
        public object CommandObject;
        public string TotalString;

        public override string ToString()
        {
            return TotalString;
        }

        public Command(string word, object target = null)
        {
            CommandWord = word;
            CommandObject = target;
            TotalString = CommandWord + (target != null ? " " + CommandObject.ToString() : "");
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            Command other = obj as Command;
            return (other != null && other.TotalString == this.TotalString);
        }
    }
}
