﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ImprovedXnaGame
{
    [Serializable]
    public class Location
    {
        public LocationID ID;
        public string Description;
        public List<Item> Items = new List<Item>();
 
        public Location(LocationID id, string description)
        {
            ID = id;
            Description = description;
        }

        internal void GetSpecialCommands(Session session, out List<Command> avails, out List<Command> recomends)
        {
            avails = new List<Command>();
            recomends = new List<Command>();
            switch (ID)
            {
                case LocationID.Splash:
                    avails.Add(new Command("begin"));
                    recomends.Add(new Command("begin"));
                    break;
                case LocationID.Laboratory:
                    if (session.Flag_SpellbookRead && !session.Flag_Step4Done)
                    avails.Add(new Command("cast summoning spell"));
                    if (session.Flag_Step4Done)
                    {
                        avails.Add(new Command("go outside"));
                        recomends.Add(new Command("go outside"));
                    }
                    
                    recomends.Add(new Command("read spellbook"));
                    recomends.Add(new Command("read Equestria Daily"));
                    break;
                case LocationID.Outside:
                    avails.Add(new Command("cast weather-control spell"));
                    avails.Add(new Command("teleport to the village"));
                    if (!session.Flag_ComfortedClover)
                       avails.Add(new Command("comfort Clover the Clever"));
                    avails.Add(new Command("empower the unicorns' force field"));
                    break;
                case LocationID.DarkCave:
                    if (!session.Flag_CastLightSpell)
                    {
                        avails.Add(new Command("cast light spell"));
                    }
                    if (session.Flag_DoorDestroyed)
                    {
                        avails.Add(new Command("go downwards"));
                    }
                    avails.Add(new Command("cast sense-life spell"));
                    break;
                case LocationID.LavaRoom:
                    if (!session.Flag_RopeConjured &&
                       (session.Flag_ApplejackSpokenTo || session.Flag_TwilightSpokenTo))
                    {
                        avails.Add(new Command("conjure rope"));
                        recomends.Add(new Command("conjure rope"));
                    }
                    if (session.Flag_FluttershyIsReady && !session.Flag_FluttershyRainbowRarityOver)
                    {
                        avails.Add(new Command("send Fluttershy, Rainbow Dash and Rarity over"));
                        recomends.Add(new Command("send Fluttershy, Rainbow Dash and Rarity over"));
                    }
                    if (session.Flag_FluttershyIsReady && session.Flag_PinkieAndApplejackAcross && !session.Flag_StarSwirlTwilightOver)
                    {
                        avails.Add(new Command("teleport over and tell Twilight to do the same"));
                        recomends.Add(new Command("teleport over and tell Twilight to do the same"));
                    }
                    if (session.Flag_FluttershyRainbowRarityOver && session.Flag_StarSwirlTwilightOver)
                    {
                        avails.Add(new Command("go downwards"));
                        recomends.Add(new Command("go downwards"));
                    }
                    break;
                case LocationID.FinalDoor:
                    if (session.Flag_AllDoorsOpen)
                    {
                        avails.Add(new Command("enter the Narrative Core"));
                    }
                    else
                    {
                        recomends.Add(new Command("examine instructions"));
                        avails.Add(new Command("press button 1"));
                        avails.Add(new Command("press button 2"));
                        avails.Add(new Command("press button 3"));
                        avails.Add(new Command("press button 4"));
                        avails.Add(new Command("press button 5"));
                        avails.Add(new Command("press button 6"));
                        avails.Add(new Command("press button 7"));
                        avails.Add(new Command("press button 8"));
                        avails.Add(new Command("press button 9"));
                        avails.Add(new Command("press button 10"));
                        avails.Add(new Command("press button 11"));
                        avails.Add(new Command("press button 12"));
                        avails.Add(new Command("press button 13"));
                        avails.Add(new Command("press button 14"));
                        avails.Add(new Command("press button 15"));
                        avails.Add(new Command("press button 16"));
                    }
                    break;
                case LocationID.FinalChoice:
                    avails.Add(new Command("cast fateseal spell"));
                    avails.Add(new Command("change my choice"));
                    recomends.Add(new Command("cast fateseal spell"));
                    recomends.Add(new Command("change my choice"));
                    break;
            }
        }
    }

    [Serializable]
    public enum LocationID
    {
        Splash,
        Laboratory,
        Outside,
        DarkCave,
        LavaRoom,
        FinalDoor,
        NarrativeCore,
        FinalChoice,
        TheEnd
    }
}
