﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Auxiliary;
using Cother;

namespace ImprovedXnaGame
{
    [Serializable]
    public partial class Session
    {
        // Utilized commands
        public List<string> UtilizedCommands = new List<string>();
        // Console output
        public List<string> ConsoleOutput = new List<string>();
        public int ConsoleTotalLines = 0;

        // Current state
        public Location CurrentLocation = null;
        public List<Item> Inventory = new List<Item>();
        public Dictionary<LocationID, Location> Locations = new Dictionary<LocationID, Location>();

        // State machine flags
        public bool Flag_SpellbookRead = false;
        public bool Flag_Step1Done = false;
        public bool Flag_Step2Done = false;
        public bool Flag_Step3Done = false;
        public bool Flag_Step4Done = false;
        public bool Flag_ComfortedClover = false;
        public bool Flag_CastLightSpell = false;
        public bool Flag_ThoughtUpon1;
        public bool Flag_SpokeToAtLeastOnePony = false;
        public FutureID Flag_ChosenFuture = FutureID.None;

        public string GetLocationDescription(Location loc)
        {
            string output = loc.Description;
            switch (loc.ID)
            {
                case LocationID.Laboratory:
                    if (loc.Items.Find((Item i) => { return i.ID == ItemID.CloverPhase1; }) != null)
                    {
                        output += " Next to the door stands your student and friend, Clover the Clever.";
                    }
                    break;
                case LocationID.DarkCave:
                    if (loc.Items.Find((Item i) => { return i.ID == ItemID.Diamond1; }) != null)
                    {
                        output += " Something glitters at the far end of the room.";
                    }
                    if (loc.Items.Find((Item i) => { return i.ID == ItemID.MetalDoor; }) != null)
                    {
                        output += " There is a locked metal door probably leading further downstairs.";
                    }
                    break;
                case LocationID.LavaRoom:
                    if (Flag_EnteredLavaRoom)
                    {
                        output += " Six ponies from the future, each of a different skin color are near you, looking confused.";
                    }
                    break;
                case LocationID.FinalChoice:
                    switch (Flag_ChosenFuture)
                    {
                        case FutureID.CallInCelestia: output += Environment.NewLine + "Chosen future: Call in the royal pony sisters to save Equestria."; break;
                        case FutureID.EmpowerRadiance: output += Environment.NewLine + "Chosen future: Empower the avatar of Friendship, forcing feelings of friendship and tolerance on everypony."; break;
                        case FutureID.LockdownTime: output += Environment.NewLine + "Chosen future: Seal the timelines, preventing time travel for all eternity."; break;
                        case FutureID.ReturnHome: output += Environment.NewLine + "Chosen future: Be returned to your laboratory and live happy ever after, leaving the future ponies to handle Discord themselves."; break;
                    }
                    break;
                case LocationID.TheEnd:
                    switch (Flag_ChosenFuture)
                    {
                        case FutureID.CallInCelestia:
                            output += Environment.NewLine + @"""My name is Celestia. I'd like to say a few words.
When I arrived with my sister a few years ago at the calling of Star Swirl the Bearded, we rejoiced, for while his sacrifice was great, it permitted us to exile Discord from Equestria forever. I was most happy.

Little did I knew that just a few years later, I'd have to exile my own sister as though she was as malevolent as Discord. I blame nopony but myself for this. Even now, I know it will take over a thousand years for Luna to be brought back to us - to me.

But do not worry, citizens of Equestria. I will not trouble you with my sorrow. The day/night cycle shall remained unperturbed and I will ensure harmony rules over Equestria for always.""";
                            break;
                        case FutureID.EmpowerRadiance:
                            output += Environment.NewLine + @"""My name is Radiance. I'd like to address everypony, please.
Today, you rejoice, for harmony has come to rule in flesh over Equestria. You will not stop rejoicing for laughter is now ingrained in your very bodies. But please, listen to me for a while.

You must realize that when Star Swirl the Bearded chose me to come to Equestria, you have lost part of your freedom. While you may feel that greed, treason and cruelty have no place in Equestria - as it is now - the possibility to engage in these was always yours until I took it away.

It is a great loss, but I have no apology to make. You may not be free, but you will be happy and Equestria will be better for it.""";
                            break;
                        case FutureID.LockdownTime:
                            output += Environment.NewLine + @"""My name is Twilight Sparkle. Everypony, please listen to me.
If we defeated Discord a few days ago, it was not only because of our friendship, but also because of Star Swirl the Bearded who saved our lives so many years ago. In the end, he chose to save Equestria by locking down the temporal strands of the fabric of the universe.

What this means is that from now on, our future is predetermined. There is no longer a way to change time, no longer a way to alter our past or our future. But our actions still have meaning. While the future is already known to some, it is still shaped by who are we and who we choose to be.

With that said, I agree wholeheartedly with Star Swirl's choice and would not have chosen otherwise myself.""";
                            break;
                        case FutureID.ReturnHome:
                            output += Environment.NewLine + @"""My name is Clover the Clever and I want to say that Star Swirl was a great stallion.

He was like a father to me. The first half of his life, he dedicated to magical research and through his study, ponies have gained many tools and spells they today take for granted. The other half of his life, he opened himself up and became a different pony. I'd never seen him happier before.

The message he would want to pass on is that, no matter the difficulties Discord's rule imposes, you may live a good life regardless of your choice, whether you dedicate it to ""pushing the boundaries of knowledge"" or you live it as a simple pony. But in these times, whatever you choose, remember to keep the Fire of Friendship strong, as Star Swirl had always done.""";
                            break;
                    }
                    output += Environment.NewLine  + Environment.NewLine + "THE END" + Environment.NewLine + Environment.NewLine + "Thank you for playing Star Swirl's Parser-Based Pony Adventure. I hope you enjoyed it. Please send me any feedback either via My Little Game Jam or to petrhudecek2010@gmail.com.";
                    break;
            }
            return output;
        }
        public void EnterLocation(Location location, bool nograying = false)
        {
            CurrentLocation = location;
            string output = GetLocationDescription(CurrentLocation);
            WriteLines(output, nograying);
        }

        public void WriteLines(string lines, bool nograying = false)
        {
            List<string> news = Primitives.GetLines(lines, ImprovedGame.rectConsoleTopInner, Library.FontConsoleNormal);
            ConsoleTotalLines += news.Count;
            ConsoleOutput.AddRange(news);
            ImprovedGame.Main.LinesAdded(nograying);
        }

        // TODO (elsewhere) implement HELP
        // Constructor (adding all locations)
        public Session()
        {
            Location splash = new Location(LocationID.Splash,
                @"Welcome to Star Swirl's Parser-Based Pony Adventure. You and I will have much fun together - well, at least ''I'' will have fun. But you'll learn about me later. For now, type ""begin"" to start the adventure or ""restore"" to load your previous (futile) attempt. Command suggestions are always listed below the input box and you may scroll through them with arrow keys.

Commands that work always are ""save"" to save your adventure to disk, ""help"" to get some more advice, ""look around"" to, you know, look around and ""exit"" to quit the game.

So let's play, puny pony, and see if you have what it takes to defeat CHAOS!!!!");
            Locations.Add(LocationID.Splash, splash);

            Location lab = new Location(LocationID.Laboratory,
                @"Around you is your laboratory. There is a number of magical and scientific instruments laying around. A spellbook lies on the table, a copy of Equestria Daily lies on the floor and a violet flask, a blue one, an orange one and a green one lie scattered around. A magical bowl is in the center of the room.");
            lab.Items.Add(new Item(ItemID.Spellbook, "Spellbook", "It contains your most powerful spells, chief among them a spell to summon the avatar of friendship.", new string[] { "read" }));
            lab.Items.Add(new Item(ItemID.UnicorniaDaily, "Equestria Daily Newspaper", @"Headline: Chaos Storm unlike any seen before approaches Pegatlantis. You might want to ""read"" it.", new string[] { "read"} ));
            lab.Items.Add(new Item(ItemID.VioletFlask, "Violet Flask", @"A vessel containing violet liquid. At some point, you might want to ""pour"" it into the bowl."));
            lab.Items.Add(new Item(ItemID.BlueFlask, "Blue Flask", @"A vessel containing blue liquid. At some point, you might want to ""pour"" it into the bowl."));
            lab.Items.Add(new Item(ItemID.OrangeFlask, "Orange Flask", @"A vessel containing orange liquid. At some point, you might want to ""pour"" it into the bowl."));
            lab.Items.Add(new Item(ItemID.GreenFlask, "Green Flask", @"A vessel containing green liquid. At some point, you might want to ""pour"" it into the bowl."));
            lab.Items.Add(new Item(ItemID.CloverPhase1, "Clover the Clever", @"Your faithful student, Clover the Clever. I think she is jealous of you. Be prudent around her."));
            Locations.Add(LocationID.Laboratory, lab);

            Location outside = new Location(LocationID.Outside,
                @"You are outside your home, Clover standing next to you. Three unicorn villages are in sight, as always, but a massive black cloud with roiling thunderstorms approaches from the east and will arrive over the first village in a matter of minutes. The villagers have already put up a force field but you suspect it won't hold.

""What can we do, Star?"" asks Clover, ""there must be something.""");
            Locations.Add(LocationID.Outside, outside);

            Location darkCave = new Location(LocationID.DarkCave,
                @"You are inside a dark cave. It's pitch black around here, but you know Clover was not teleported along. You are alone.");
            Locations.Add(LocationID.DarkCave, darkCave);

            Location lavaRoom = new Location(LocationID.LavaRoom,
                @"You are in a huge room. A lava river, about five meters wide, separates the two parts of the room. On the other side, there is a strong, but open door. The passage behind leads further downstairs.");
            lavaRoom.Items.Add(new Item(ItemID.Pony_Twilight, "Twilight Sparkle", "A violet unicorn from the future. She looks smart and might be able to give you advice.", new string[] { "talk to" }));
            lavaRoom.Items.Add(new Item(ItemID.Pony_Applejack, "Applejack", "An orange earth pony from the future. She looks like she could jump over the river without help of magic.", new string[] { "talk to" }));
            lavaRoom.Items.Add(new Item(ItemID.Pony_PinkiePie, "Pinkie Pie", "A pink earth pony from the future. A little hyperactive.", new string[] { "talk to" }));
            lavaRoom.Items.Add(new Item(ItemID.Pony_Rarity, "Rarity", "A white unicorn from the future. She doesn't look like much of a help in this scenario.", new string[] { "talk to" }));
            lavaRoom.Items.Add(new Item(ItemID.Pony_Fluttershy, "Fluttershy", "A yellow pegasus from the future. You worry she might not be brave enough to fly across the river.", new string[] { "talk to" }));
            lavaRoom.Items.Add(new Item(ItemID.Pony_RainbowDash, "Rainbow Dash", "A cyan pegasus from the future. On her own and in this heat, she won't be able to fly everypony across the river, though she should be able to fly over easily herself.", new string[] { "talk to" }));
            Locations.Add(LocationID.LavaRoom, lavaRoom);

            Location finalDoor = new Location(LocationID.FinalDoor, @"You stand in front of a massive solid gate made from an unrecognizable material. A keypad is engraved in the gate and there are some instructions next to it.");
            finalDoor.Items.Add(new Item(ItemID.Instructions, "Instructions", @"To gain access to the Narrative Core, you must unlock all six doors. If you make a mistake, make sure to press the Reset button.

button 1: trap
button 2: unlocks lock 4, locks door 5
button 3: unlocks door 5, locks door 4
button 4: unlocks door 1, locks door 5
button 5: unlocks door 5, locks door 1
button 6: unlocks door 2, locks door 4
button 7: unlocks door 6
button 8: unlocks door 3, locks door 4
button 9: unlocks door 2, locks door 3
button 10: unlocks door 2, locks door 1
button 11: unlocks door 6, locks door 5
button 12: unlocks door 3, locks door 4
button 13: trap
button 14: unlocks door 1, locks door 2
button 15: unlocks door 5, locks door 6
button 16: reset (locks all doors)")); 
            finalDoor.Items.Add(new Item(ItemID.FinalDoor, "Gate", @"You cannot bypass the gate in any way except by solving the instructions puzzle. It is made from an alloy of indestructible metal you have not seen before."));
            Locations.Add(LocationID.FinalDoor, finalDoor);

            Location narrativeCore = new Location(LocationID.NarrativeCore,
@"You stand in a medium-sized oval room with four colorful screens. As you watch the images, you realize these are four different futures, four different ways the life in Equestria may continue and it is you, a single pony, cut off from the surface, who must decide what shall come to pass. 

The storms of chaos and Discord will be eliminated in every future but still, each way is fundamentally different from all other. Choose carefully, for the fate of Equestria is in your hooves.

Using the ""save"" command might also be appropriate at this time.");
            narrativeCore.Items.Add(new Item(ItemID.Monitor_Western, "West Screen", @"Viewed Future:
You expend your lifeforce to lock down the timeline forever! The narrative, from now, will be fixed. Time travel and time stopping will not ever be possible again, not even Discord will be able to use them, because while he's outside the logic, he still has to obey the rules of storyline.

Without the power to travel through time, Discord will be isolated and weak. In the future, Twilight Sparkle and her Ponyville friends will encase him in stone and life in Equestria will continue with harmony.

Star Swirl the Bearded wing will never be constructed because time spells will not exist but a statue representing Sacrifice with your likeness will be erected in Canterlot Gardens."));
            narrativeCore.Items.Add(new Item(ItemID.Monitor_Northern, "North Screen", @"Viewed Future:
You expend your lifeforce to reach beyond the existing narrative and call upon Princess Celestia and Princess Luna to come sooner than predicted. Elements of Harmony will be used simultaneously in your time and in the future (insofar as the word ""simultaneously"" makes sense in the context) to exile Discord forever.

In addition, the premature arrival of the royal pony sisters will bring harmony and friendship to Equestria hundreds of years prior to the original timeline. On the other hand, this also means Luna will be exiled to the moon much earlier and her imprisonment will last that much longer.

Your heroic sacrifice and instrumental role in the defeat of Discord shall be immortalized when your name will be given to one of the most secure sections of the Canterlot Archives - the Star Swirl the Bearded wing."));
            narrativeCore.Items.Add(new Item(ItemID.Monitor_Eastern, "East Screen", @"Viewed Future:
You transfer your lifeforce to Radiance, the personification of Friendship and so empower her to descend to Equestria in flesh. As friendship is much stronger than discord, she will defeat Discord immediately and restore happiness across the land.

However, the royal pony sisters will never become rulers of Equestria as Radiance assumes the position herself in the name of greater good. Her awesome powers will enforce the ideals of Harmony - friendship, kindness, generosity, laughter, loyalty and honesty - upon everypony and it will become physically impossible to act or think against these principles.

Time spells will be forbidden in the interest of preserving the status quo of bliss, but your name will be forever remembered as the name of the hero who ushered Equestria into a new golden era."));
            narrativeCore.Items.Add(new Item(ItemID.Monitor_Southern, "South Screen", @"Viewed Future:
You refuse to choose a future of Equestria - that responsibility should not fall to a single pony. Instead, you request the Narrative Core to transport you to your laboratory.

You will lose interest in higher magical research and in saving Equestria from Chaos. Instead, you will seek personal happiness. Clover the Clever will introduce you to the world and you will make many friends, ponies, who will make your life complete. You will live a full life but you will never come to terms with the fact that you chose personal gain over the good of Equestria.

Discord will eventually be defeated by Twilight and her pony friends in the future and Celestia and Luna in the present, but hundreds more years will pass in a state of chaos.

Your name will be immortalized in the history books as an inventor of tens of spells per year until, inexplicably, your scientific career came to an end. This is the canonical Friendship is Magic narrative."));
            Locations.Add(LocationID.NarrativeCore, narrativeCore);

            Location finalChoice = new Location(LocationID.FinalChoice, @"You have chosen a future. If you are truly sure this is how you want this adventure to end, cast a fateseal spell. Otherwise, type ""change my choice"".");
            Locations.Add(LocationID.FinalChoice, finalChoice);

            Location theEnd = new Location(LocationID.TheEnd, @"");
            Locations.Add(LocationID.TheEnd, theEnd);
        }


        public void GetAvailableCommands(out List<Command> availableCommands, out List<Command> recommendedCommands)
        {
            availableCommands = new List<Command>();
            recommendedCommands = new List<Command>();
            Location l = CurrentLocation;
           
            // Location-based
            List<Command> avails; List<Command> recomends;
            l.GetSpecialCommands(this, out avails, out recomends);
            availableCommands.AddRange(avails);
            recommendedCommands.AddRange(recomends);
            if (Flag_SpellbookRead && !Flag_Step1Done) recommendedCommands.Add(new Command("pour in blue flask"));

            // Location-based item-based
            foreach (Item item in l.Items)
            {
                List<string> permittedCommands = new List<string>(item.PermittedCommands);
                permittedCommands.Add("examine");
                foreach (string s in permittedCommands)
                {
                    Command c = new Command(s, item);
                    availableCommands.Add(c);
                }
            }

            // Special
            if (Flag_DoorDestroyed && !Flag_ThoughtUpon1)
            {
                availableCommands.Add(new Command("think upon Clover"));
            }

            // Generals
            if (Inventory.Count > 0)
            {
                availableCommands.Add(new Command("inventory"));
            }
            // Save
            if (l.ID != LocationID.Splash)
            {
                availableCommands.Add(new Command("look around"));
                availableCommands.Add(new Command("save"));
                if (!UtilizedCommands.Contains("save") && !UtilizedCommands.Contains("restore")) recommendedCommands.Add(new Command("save"));
            }
            availableCommands.Add(new Command("restore"));
            availableCommands.Add(new Command("exit"));
            availableCommands.Add(new Command("toggle fullscreen"));
            availableCommands.Add(new Command("clear"));
            availableCommands.Add(new Command("help"));
            // TODO recommend restore, when possible
        }



      
    }
    public enum FutureID
    {
        LockdownTime,
        CallInCelestia,
        EmpowerRadiance,
        ReturnHome,
        None
    }
}
